package cn.iocoder.yudao.module.shipment.listener;


import cn.iocoder.yudao.framework.apollo.core.event.box.BoxCheckOrderApprovalEvent;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxApprovalDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.enums.BoxApprovalTypeEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxApproval.BoxApprovalService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.module.order.enums.OrderApprovalTypeResultEnum.*;

/**
 * 提供订单最新的进行中审批任务
 *
 * @author zhengYi
 */
@Component("BoxCheckOrderApprovalListener")
@AllArgsConstructor
@Slf4j
public class BoxCheckOrderApprovalListener {

    private final BoxApprovalService boxApprovalService;
    private final BoxService boxService;


    /**
     * 提供订单是否存在进行中的审批校验
     *
     * @param event 订单信息与结果
     */
    @EventListener(BoxCheckOrderApprovalEvent.class)
    public void checkOrderApproval(BoxCheckOrderApprovalEvent event) {
        BoxDO boxDO = boxService.getOne(new LambdaQueryWrapper<BoxDO>().eq(BoxDO::getSelfNo, event.getSelfNo()));
        if (boxDO == null) {
            return;
        }
        List<BoxApprovalDO> boxApprovalDOList = boxApprovalService.checkOrderProcessingApproval(boxDO.getId(), event.getOrderId());
        if (Objects.isNull(boxApprovalDOList)) {
            return;
        }
        for (BoxApprovalDO boxApprovalDO : boxApprovalDOList) {
            BoxApprovalTypeEnum typeEnum = BoxApprovalTypeEnum.getApprovalTypeByTypeValue(boxApprovalDO.getApprovalType());
            if (Objects.isNull(typeEnum)) {
                continue;
            }
            event.setIsExists(true);
            switch (typeEnum) {
                case PRELOAD:
                    //预装
                    event.setAuditType(pre_install_processing.getResult());
                    event.setAuditResult(pre_install_processing.getDesc());
                    return;
                case SEALING_CABINET:
                    //装柜
                    event.setAuditType(closure_processing.getResult());
                    event.setAuditResult(closure_processing.getDesc());
                    return;
                case UNLOAD_CABINET:
                    // 卸柜
                    event.setAuditType(unloading_cabinet_processing.getResult());
                    event.setAuditResult(unloading_cabinet_processing.getDesc());
                    return;
                case PRELOAD_UMPIRE:
                    //预装反审
                    event.setAuditType(pre_install_back_processing.getResult());
                    event.setAuditResult(pre_install_back_processing.getDesc());
                    return;
                case CUSTOMS_FULL_REFUND:
                    //报关全退
                    event.setAuditType(customs_exit_processing.getResult());
                    event.setAuditResult(customs_exit_processing.getDesc());
                    return;
                case UNLOAD_UMPIRE:
                    //卸柜反审
                    event.setAuditType(unloading_cabinet_back_processing.getResult());
                    event.setAuditResult(unloading_cabinet_back_processing.getDesc());
                    return;
                case SEALING_CABINET_UMPIRE:
                    //封柜反审
                    event.setAuditType(closure_back_processing.getResult());
                    event.setAuditResult(closure_back_processing.getDesc());
                    return;
                case AIR_SORTING:
                    // 排单分拣
                    event.setAuditType(sorting_processing.getResult());
                    event.setAuditResult(sorting_processing.getDesc());
                    return;
                case AIR_SORTING_BACK:
                    // 排单分拣反审
                    event.setAuditType(sorting_back_processing.getResult());
                    event.setAuditResult(sorting_back_processing.getDesc());
                    return;
                case AIR_TO_WAREHOUSE:
                    // 到仓
                    event.setAuditType(warehoused_processing.getResult());
                    event.setAuditResult(warehoused_processing.getDesc());
                    return;
                case AIR_TO_WAREHOUSE_BACK:
                    // 到仓反审
                    event.setAuditType(warehoused_back_processing.getResult());
                    event.setAuditResult(warehoused_back_processing.getDesc());
                    return;
                case AIR_SHIPMENT:
                    // 出货
                    event.setAuditType(shipment_processing.getResult());
                    event.setAuditResult(shipment_processing.getDesc());
                    return;
                case AIR_SHIPMENT_BACK:
                    // 出货反审
                    event.setAuditType(shipment_back_processing.getResult());
                    event.setAuditResult(shipment_back_processing.getDesc());
                    return;
                case CLEARANCE_BACK:
                    // 撤销清关
                    event.setAuditType(clearance_back_processing.getResult());
                    event.setAuditResult(clearance_back_processing.getDesc());
                    return;
                default:
                    event.setIsExists(false);
            }
        }
    }


}
